import { ServiceSchema } from 'moleculer';
import { SubjectRawRule, AnyMongoAbility, ExtractSubjectType, Subject, MongoQuery } from '@casl/ability';

type DefaultUser = {
    id: string;
};
type AnyRules = SubjectRawRule<any, any, any>[];
type AbilityConsumerMethods<TUser extends object = DefaultUser> = {
    abilityFor: (user: TUser) => Promise<AnyMongoAbility>;
};

/**
 * Creates a mixin that adds an action to provide CASL rules for other services to query.
 *
 * @param rulesFor A function that takes your desired entity and returns raw CASL rules.
 * @returns Mixin
 */
declare function createCASLAbilityProvider<TUser extends object = DefaultUser, TMeta extends object = object>(rulesFor: (user: TUser) => AnyRules): Partial<ServiceSchema>;

/**
 * Creates a mixin that services can use to get Abilities for an entity,
 * from another service that provides the rules.
 *
 * @param {string} [service] The name of the service that has CASL Ability Provider mixin. Default 'auth'
 * @returns Mixin
 */
declare function createCASLAbilityConsumer<TUser extends object = DefaultUser, TRules extends SubjectRawRule<string, ExtractSubjectType<Subject>, MongoQuery>[] = AnyRules>(service?: string): Partial<ServiceSchema & {
    methods: AbilityConsumerMethods<TUser>;
}>;

export { type AbilityConsumerMethods, type AnyRules, type DefaultUser, createCASLAbilityConsumer, createCASLAbilityProvider };
